/******************************************************************************
*
* project name:    TI-Chess / BookBuilder
* file name:       hash.c
* initial date:    04/10/2002
* author:          thomas.nussbaumer@gmx.net
* description:     hashcode generation for bookbuilder
*
* 2 functions are exported:
*
* unsigned long long CalcHashcode(char* board);
* .... calculates hashcode from 64-byte long board
*
* char* Hashcode2String(unsigned long long hashcode);
* .... converts hashcode to string
*
* $Id: hash.c,v 1.2 2002/10/23 21:06:08 tnussb Exp $
*
******************************************************************************/

unsigned long x[55] = {
       1410651636UL, 3012776752UL, 3497475623UL, 2892145026UL, 1571949714UL,
       3253082284UL, 3489895018UL,  387949491UL, 2597396737UL, 1981903553UL,
       3160251843UL,  129444464UL, 1851443344UL, 4156445905UL,  224604922UL,
       1455067070UL, 3953493484UL, 1460937157UL, 2528362617UL,  317430674UL,
       3229354360UL,  117491133UL,  832845075UL, 1961600170UL, 1321557429UL,
        747750121UL,  545747446UL,  810476036UL,  503334515UL, 4088144633UL,
       2824216555UL, 3738252341UL, 3493754131UL, 3672533954UL,   29494241UL,
       1180928407UL, 4213624418UL,   33062851UL, 3221315737UL, 1145213552UL,
       2957984897UL, 4078668503UL, 2262661702UL,   65478801UL, 2527208841UL,
       1960622036UL,  315685891UL, 1196037864UL,  804614524UL, 1421733266UL,
       2017105031UL, 3882325900UL,  810735053UL,  384606609UL, 2393861397UL
};

unsigned long      y[55];
short              jjj = 0;
short              kkk = 0;
short              init_random = 1;
short              already_initialized = 0;
unsigned long long hashcodes[12][64];


//=============================================================================
// 32 bit random number generator
//=============================================================================
static unsigned long Random32(void) {
    unsigned long  ul;

    if (init_random) {
       short i;

       init_random = 0;
       for (i = 0; i < 55; i++) y[i] = x[i];
       jjj = 24 - 1;
       kkk = 55 - 1;
    }

    ul = (y[kkk] += y[jjj]);
    if (--jjj < 0) jjj = 55 - 1;
    if (--kkk < 0) kkk = 55 - 1;
    return(ul);
}



//=============================================================================
// setup the hashcode table by assigning each square/piece/side combination
// an own 64bit hashcode
//=============================================================================
static void InitHashCodes(void) {
    short  figure, sq;
    unsigned long long result;

    init_random = 1;

    for (figure = 0; figure < 12; figure++) {
        for (sq = 0; sq < 64; sq++) {
            result = Random32();
            result <<= 32;
            result |= Random32();
            hashcodes[figure][sq] = result;
        }
    }
}



//=============================================================================
// the hashcode is generated by looping through all squares and XORing all
// hashcodes of the square/piece/color combinations found
//=============================================================================
unsigned long long CalcHashcode(char* board) {
    unsigned long long act_hashkey = 0;
    short  sq;
    short  sq2;
    char   fig;

    // initialized hashcodes if necessary
    if (!already_initialized) {
        InitHashCodes();
        already_initialized = 1;
    }

    for (sq=0,sq2=0; sq<BOARD_SIZE; sq++,board++) {
        fig = *board;
        if (fig == OUTSIDE) continue;
        if (fig == EMPTY) {
            sq2++;
            continue;
        }
        if (fig < 0) fig = -fig + 6;
        fig--;
        act_hashkey ^= hashcodes[(unsigned char)fig][sq2];
        sq2++;
    }
    return act_hashkey;
}

char _hashcode_buffer_[17];
//=============================================================================
// converts hashcode to string
//=============================================================================
char* Hashcode2String(unsigned long long hashcode) {
    unsigned long hash_upper = (hashcode >> 32) & 0xffffffffUL;
    unsigned long hash_lower = hashcode & 0xffffffffUL;

    sprintf(_hashcode_buffer_,"%08lX%08lX",hash_upper,hash_lower);
    return _hashcode_buffer_;
}

//#############################################################################
//###################### NO MORE FAKES BEYOND THIS LINE #######################
//#############################################################################
//
//=============================================================================
// Revision History
//=============================================================================
//
// $Log: hash.c,v $
// Revision 1.2  2002/10/23 21:06:08  tnussb
// changes to support PGN input instead of idiotic from-to notation
//
// Revision 1.1  2002/10/08 17:40:13  tnussb
// initial check-in
//
//
